use std::{fs::{self, File}, path::PathBuf};

use anyhow::Context;
use serde::{de, Deserialize, Serialize};

#[derive(Deserialize, Serialize, Debug)]
struct Config {
    key: Option<Vec<u8>>
}

impl Default for Config {
    fn default() -> Self {
        Self { key: Default::default() }
    }
}

#[derive(Debug)]
pub struct ConfigProvider {
    config: Config,
    base_path: PathBuf
}

impl ConfigProvider {
    pub fn new() -> anyhow::Result<ConfigProvider> {
        let mut base_path = dirs::preference_dir().with_context(|| "Failed to find your preference directory")?;
        base_path.push(env!("CARGO_PKG_NAME"));

        fs::create_dir_all(&base_path)?;

        base_path.push("config.json");

        if !fs::exists(&base_path)? {
            let mut fd = File::create(&base_path)?;
            serde_json::to_writer(&mut fd, &Config::default())?;
            drop(fd);
        }

        let mut fd = File::open(&base_path)?;
        let config: Config = serde_json::from_reader(&mut fd)?;
        drop(fd);

        Ok(ConfigProvider { config, base_path })
    }

    fn commit(&mut self) -> anyhow::Result<()> {
        let mut fd = File::create(&self.base_path)?;
        serde_json::to_writer(&mut fd, &self.config)?;
        drop(fd);

        Ok(())
    }

    pub fn get_key(&self) -> Option<Vec<u8>> {
        return self.config.key.clone();
    }

    pub fn set_key(&mut self, key: &Vec<u8>) -> anyhow::Result<()> {
        self.config.key = Some(key.clone());
        self.commit()?;

        Ok(())
    }
}
